# Minterpy JOSS Paper Data

This is a readme file for the data used in the publication
"Minterpy: Multivariate interpolation in Python" submitted to the
Journal of Open Source Software (JOSS).

This archive contains the following:

- `README.md`: this file
- `plot/`
  - `convergence.png`: The convergence plot
  - `create_plot.py`: The python script to create the convergence plot
- `linf-errors/`: Convergence data obtained from different package
  approximating Runge function in dimension three and four with parameter 1.0.
  Each csv files contains two-column data, the first is the number of
  coefficients or data points and the second is the infinity norm error.
  The error is computed based on 1'000'000 random points in [-1, 1]^m, where
  `m` is either 3 or 4. A common seed number is used to generate the points.
  - `chaospy/`: Convergence data obtained from ChaosPy
    - `errors-chaospy-dim_3-nmin_000-nmax_020-runge_param_1_0.csv`: Convergence
      of Legendre polynomials approximating 3D Runge function in tensorial grid
      with polynomial degrees 0 to 20.
    - `errors-chaospy-dim_4-nmin_000-nmax_015-runge_param_1_0.csv`: Convergence
      of Legendre polynomials approximating 4D Runge function in tensorial grid
      with polynomial degrees 0 to 15.
  - `equadratures/`: Convergence data obtained from equadratures
    - `errors-equadratures-dim_3-nmin_000-nmax_020-runge_param_1_0.csv`:
      Convergence of Legendre polynomials approximating 3D Runge function in
      tensorial grid with polynomial degrees 0 to 20.
    - `errors-equadratures-dim_4-nmin_000-nmax_010-runge_param_1_0.csv`:
      Convergence of Legendre polynomials approximating 4D Runge function in 
      tensorial grid with polynomial degrees 0 to 10.
  - `interpn/`: Convergence data obtained from SciPy (points are generated 
    according to Chebyshev-Lobatto nodes)
    - `errors-interpn-dim_3-linear-max_1d_100-runge_param_1_0.csv`: Convergence
      of linear interpolation approximating 3D Runge function in tensorial grid
      with 100 maximum number of points in one dimension.
    - `errors-interpn-dim_3-pchip-max_1d_100-runge_param_1_0.csv`:
      Convergence of piecewise cubic Hermite interpolating polynomials
      approximating 3D Runge function in tensorial grid with 100 maximum number
      of points in one dimension.
    - `errors-interpn-dim_3-nearest-max_1d_100-runge_param_1_0.csv`:
      Convergence of nearest point interpolation approximating 3D Runge
      function in tensorial grid with 100 maximum number of points
      in one dimension.
    - `errors-interpn-dim_4-linear-max_1d_50-runge_param_1_0.csv`: Convergence
      of linear interpolation approximating 4D Runge function in tensorial grid
      with 50 maximum number of points in one dimension.
    - `errors-interpn-dim_4-pchip-max_1d_50-runge_param_1_0.csv`: Convergence
      of piecewise cubic Hermite interpolating polynomials approximating 4D 
      Runge function in tensorial grid with 50 maximum number of points
      in one dimension.
    - `errors-interpn-dim_4-nearest-max_1d_50-runge_param_1_0.csv`: Convergence
      of nearest point interpolation approximating 4D Runge function
      in tensorial grid with 50 maximum number of points in one dimension.
  - `minterpy`: Convergence data obtained from Minterpy
    - `errors-minterpy-dim_3-lp_1.0-nmin_000-nmax_100-runge_param_1.0.csv`:
      Convergence of Minterpy interpolating polynomials approximating 3D Runge
      function with lp-degree 1.0 and polynomial degrees 0 to 100.
    - `errors-minterpy-dim_3-lp_2.0-nmin_000-nmax_070-runge_param_1_0.csv`:
      Convergence of Minterpy interpolating polynomials approximating 3D Runge
      function with lp-degree 2.0 and polynomial degrees 0 to 70.
    - `errors-minterpy-dim_3-lp_inf-nmin_000-nmax_055-runge_param_1_0.csv`:
      Convergence of Minterpy interpolating polynomials approximating 3D Runge
      function with lp-degree inf and polynomial degrees 0 to 55.
    - `errors-minterpy-dim_4-lp_1.0-nmin_000-nmax_075-runge_param_1.0.csv`:
      Convergence of Minterpy interpolating polynomials approximating 4D Runge
      function with lp-degree 1.0 and polynomial degrees 0 to 75.
    - `errors-minterpy-dim_4-lp_2.0-nmin_000-nmax_045-runge_param_1_0.csv`:
      Convergence of Minterpy interpolating polynomials approximating 4D Runge
      function with lp-degree 2.0 and polynomial degrees 0 to 45.
    - `errors-minterpy-dim_4-lp_inf-nmin_000-nmax_040-runge_param_1_0.csv`:
      Convergence of Minterpy interpolating polynomials approximating 4D Runge
      function with lp-degree inf and polynomial degrees 0 to 40.
  - `ndsplines`: Convergence data obtained from ndsplines (points are generated 
    according to Chebyshev-Lobatto nodes)
    - `errors-ndsplines-dim_3-linear-max_1d_100-runge_param_1_0.csv`:
      Convergence of linear splines approximating 3D Runge function in
      tensorial grid with 100 maximum number of points in one dimension. 
    - `errors-ndsplines-dim_3-cubic-max_1d_100-runge_param_1_0.csv`:
      Convergence of cubic splines approximating 3D Runge function in tensorial
      grid with 100 maximum number of points in one dimension.
    - `errors-ndsplines-dim_3-quintic-max_1d_100-runge_param_1_0.csv`:
      Convergence of quintic splines approximating 3D Runge function in
      tensorial grid with 100 maximum number of points in one dimension.
    - `errors-ndsplines-dim_4-linear-max_1d_50-runge_param_1_0.csv`:
      Convergence of linear splines approximating 4D Runge function in
      tensorial grid with 50 maximum number of points in one dimension.
    - `errors-ndsplines-dim_4-cubic-max_1d_50-runge_param_1_0.csv`:
      Convergence of cubic splines approximating 4D Runge function in tensorial
      grid with 50 maximum number of points in one dimension.
    - `errors-ndsplines-dim_4-quintic-max_1d_50-runge_param_1_0.csv`:
      Convergence of quintic splines approximating 4D Runge function in
      tensorial grid with 50 maximum number of points in one dimension.
- `scripts`
  - `run_chaospy.py`: Script to generate convergence data from ChaosPy
  - `run_equadratures_err.py`: Script to generate model from equadratures
  - `run_equadratures_fit.py`: Script to generate convergence data from
    equadratures
  - `run_interpn.py`: Script to generate convergence data from SciPy
  - `run_minterpy.py`: Script to generate convergence data from minterpy
  - `run_ndsplines.py`: Script to generate convergence data from ndsplines
  - `utils.py`: Common utility module

## Requirements

The following are the Python package requirements to conduct the experiment:

- `minterpy==0.3.0`
- `ndsplines==0.2.0`
- `chaospy==4.3.17`
- `equadratures==10` (this version of equadratures is incompatible with NumPy
  v2)
- `scipy==1.13.1`
- `click==8.1.8`
- `tqdm==4.67.1`
